/*------------------------------------------------------------------
SpacerImage class, using C#
Written by Peter L. Blum (www.PeterBlum.com)
Free to distribute and reuse. This web control is intended to assist
all users throughout the ASP.NET community build common controls.
Please share it. In order to be successful, all developers should
maintain the same namespace and assembly so web application developers
will only need one of these controls on their sites.

Release: July 29, 2002

--- OVERVIEW -------------------------------------------------------
Most web sites employ a "spacer", a 1x1 pixel white image file that is used to form
gaps between other objects simply by using this text:
<img src="spacer.gif" width=[gapwidth] height=[gapheight] />

While it's easy to enter this each time, your User Controls and distributed Custom Controls
may also need a spacer. They must demand a specific filename and path to the image file.

The web control SpacerImage builds the standard HTML code for spacers. Plus it defines its
path to the spacer file globally, in the SpacerImage.xGlobalImageURL static property. 
Start using SpacerImage in place of other ways to write the spacer. Then assign the image file once, 
on SpacerImage.xGlobalImageURL when your application starts up. 
Every custom control that uses SpacerImage benefits from this.

Hopefully, Microsoft will adapt this idea into a future release of the .Net Framework so
every programmer uses it automatically.

By default, the file path is "/aspnet_client/spacer.gif". (/aspnet_client/ is a predefined folder 
in your web application used to contain common files retrieved by the browser. The image file 
fits that requirement.)

SpacerImage is inherited from System.Web.UI.WebControls.Image. So it inherits the Height and
Width properties, which you use to dictate spacing. These default to 1 pixel each within
the SpacerImage control.

 ----- INSTALLING INTO YOUR PROJECTS ----------------------------------------
 This file came packaged within a Zip file. If you do not intend to work with the source
 code, you can add the SpacerControl.DLL to your Visual Studio toolbox.
 
 ----- WORKING WITH THE SOURCE CODE ----------------------------------------
Follow these instructions if you want to use the source code. 
This file should come with a project titled SpacerControl.csproj. If it is not available
 or is causing problems, consider steps 1-3. All users should continue at step 4.
 1. Create a C# project for a Web Control Library. Recommended name is SpacerControl.
    Confirm that the Assembly will use the same name by selecting the Project in 
    Solution Explorer and choosing Properties. While in Properties, assign the default
    namespace to "PeterBlum.SpacerControl". (This makes the toolbox icons work.)
    
    NOTE: You should maintain the same namespace and assembly so web application developers
    will only need one of these controls on their sites.
 
 2. Add these files to the project: (File; Add Existing Item.)
    SpacerControl.cs - this file.
    SpacerImage.bmp - image for the toolbox. Also open the Properties for this file
      in Solution Explorer and change Build Action to Embedded Resource.
    AssemblyInfo.cs

 3. Your project must have these References: 
    System, System.Drawing, System.Web
    
 4. Compile the SpacerControl project to create an assembly named SpacerControl.DLL.
    This project makes a namespace called PeterBlum.SpacerControl.
    
 5. If you like, add the SpacerImage to your Toolbox. (Open the toolbox, select a
    tab where it will go, right click and choose Customize Toolbox. Locate the SpacerControl.DLL.)
    
 ----- ADDING THE SPACERCONTROL TO A WEB APPLICATION ---------------------
 Before it can be used in a web application, your web app project must reference this control.
 1. In your Web Application project, Add Reference to the SpacerControl.DLL assembly (or project).
 
 2. Supply an image file for the SpacerImage control. By default, SpacerImage looks 
    in /aspnet_client/spacer.gif. If you do not have an image file, copy the file 'spacer.gif' 
    from this package to the /aspnet_client folder. If you do have a file, you can either 
    copy it to the /aspnet_client folder or point SpacerImage.xGlobalImageURL to it in your 
    global.asax's Application_Start method:

   protected void Application_Start(object sender, EventArgs e)
   {
      SpacerImage.xGlobalImageURL = [URL to your file];
   }

 ----- ADDING THE CONTROL TO AN ASP.NET PAGE ----------------------------
 If you add the control from the Toolbox, it automatically handles steps 1-2.
 
 1. To add this control to your ASP.NET page, add this header line:
 <%@ Register TagPrefix="spc" Namespace="PeterBlum.SpacerControl" Assembly="SpacerControl" %>
 
 2. In a <FORM> area, add the control itself:
 <spc:SpacerImage id="[YourControlID]" runat=server />
         
    It defaults to a 1x1 pixel image. If you want to stretch your image (which is a common
    way to fill space to an exact pixel size), set the Width and Height properties.
    
 <spc:SpacerImage id=[YourControlID] runat=server Width="10px" Height="5px" />

------------------------------------------------------------------*/
using System;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.ComponentModel;

[assembly:TagPrefix("PeterBlum.SpacerControl", "Spc")]

namespace PeterBlum.SpacerControl
{
/* ----- CLASS SpacerImage -------------------------------------------------------- */
/// <remarks>
/// SpacerImage builds the standard HTML code used to form very small vertical and horizontal
/// gaps: <img src="spacer.gif" width="1" height="1" />
/// The program is that every application stores its 1x1 pixel image file under a different name or path.
/// This control makes that information into a static property.
/// Thus an application can assign its path to the file once, via SpacerImage.xGlobalImageURL.
/// The path defaults to "/aspnet_client/spacer.gif"
/// </remarks>
   [ToolboxData("<{0}:SpacerImage runat=server></{0}:SpacerImage>")]
   public class SpacerImage : System.Web.UI.WebControls.Image
   {
// ---- PROPERTIES -----------------------------------------------------------------
/// <summary>
/// xGlobalImageURL is a static property that is a path to an image file used as the spacer.
/// It should be assigned when the application starts up. If you lack an image file,
/// create a gif file that is 1x1 pixels and is white.
/// Defaults to the root of your current site with the file name "/aspnet_client/spacer.gif".
/// </summary>
      [Browsable(false)]
      static public string xGlobalImageURL
      {
         get { return SpacerImage.fGlobalImageURL; }
         set { SpacerImage.fGlobalImageURL = value; }
      }
      static private string fGlobalImageURL = "/aspnet_client/spacer.gif";  // default file is located in the root of your app

// -------------------------------------------------------------------------
/// <summary>
/// ImageUrl is overriden to hide it from the Properties window and ASP.NET streaming.
/// Use xGlobalImageURL instead.
/// </summary>
      [Browsable(false)]
      [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
      public override string ImageUrl
      {
         get { return base.ImageUrl; }
         set { base.ImageUrl = value; }
      }

// -------------------------------------------------------------------------
/// <summary>
/// Height is the property on Image. Here it gets a 1 pixel default.
/// </summary>
      [DefaultValue("1px")]
      public override Unit Height 
      {
         get{ return base.Height; }
         set{ base.Height = value; }
      }

// -------------------------------------------------------------------------
/// <summary>
/// Width is the property on Image. Here it gets a 1 pixel default.
/// </summary>
      [DefaultValue("1px")]
      public override Unit Width 
      {
         get{ return base.Width; }
         set{ base.Width = value; }
      }

// ------ CONSTRUCTORS ---------------------------------------------------

      public SpacerImage(string pHeight, string pWidth) : base()
      {
         Height = new Unit(pHeight);
         Width = new Unit(pWidth);
         this.ImageUrl = xGlobalImageURL;
      }  // constructor

/// <summary>
/// This constructor follows the standard format expected by ASP.NET to create
/// control instances.
/// </summary>
      public SpacerImage() : base()
      {
         Height = new Unit("1px");
         Width = new Unit("1px");
         this.ImageUrl = xGlobalImageURL;
      }  // constructor
   
   }  // class SpacerImage

}  // namespace PeterBlum.SpacerControl
